@@ -24,13 +24,14 @@
 #include "gappinfo.h"
 #include "glibintl.h"
 #include <gioerror.h>
+#include <gfile.h>
 
 #include "gioalias.h"
 
 /**
  * SECTION:gappinfo
  * @short_description: Application information and launch contexts
- * @include: gio.h
+ * @include: gio/gio.h
  * 
  * #GAppInfo and #GAppLaunchContext are used for describing and launching 
  * applications installed on the system. 
@@ -406,6 +407,11 @@ g_app_info_get_icon (GAppInfo *appinfo)
  * can fail to start if it runs into problems during startup. There is
  * no way to detect this.
  *
+ * Some URIs can be changed when passed through a GFile (for instance
+ * unsupported uris with strange formats like mailto:), so if you have
+ * a textual uri you want to pass in as argument, consider using
+ * g_app_info_launch_uris() instead.
+ * 
  * Returns: %TRUE on successful launch, %FALSE otherwise. 
  **/
 gboolean
@@ -498,7 +504,7 @@ g_app_info_launch_uris (GAppInfo           *appinfo,
 
   iface = G_APP_INFO_GET_IFACE (appinfo);
 
-  return (* iface->launch) (appinfo, uris, launch_context, error);
+  return (* iface->launch_uris) (appinfo, uris, launch_context, error);
 }
 
 
@@ -523,6 +529,50 @@ g_app_info_should_show (GAppInfo *appinfo)
   return (* iface->should_show) (appinfo);
 }
 
+/**
+ * g_app_info_launch_default_for_uri:
+ * @uri: the uri to show
+ * @launch_context: an optional #GAppLaunchContext.
+ * @error: a #GError.
+ *
+ * Utility function that launches the default application 
+ * registered to handle the specified uri. Synchronous I/O
+ * is done on the uri to detext the type of the file if
+ * required.
+ * 
+ * Returns: %TRUE on success, %FALSE on error.
+ **/
+gboolean
+g_app_info_launch_default_for_uri (const char         *uri,
+				   GAppLaunchContext  *launch_context,
+				   GError            **error)
+{
+  GAppInfo *app_info;
+  GFile *file;
+  GList l;
+  gboolean res;
+
+  file = g_file_new_for_uri (uri);
+  app_info = g_file_query_default_handler (file, NULL, error);
+  g_object_unref (file);
+  if (app_info == NULL)
+    return FALSE;
+
+  /* Use the uri, not the GFile, as the GFile roundtrip may
+   * affect the uri which we don't want (for instance for a
+   * mailto: uri).
+   */
+  l.data = (char *)uri;
+  l.next = l.prev = NULL;
+  res = g_app_info_launch_uris (app_info, &l,
+				launch_context, error);
+
+  g_object_unref (app_info);
+  
+  return res;
+}
+
+
 G_DEFINE_TYPE (GAppLaunchContext, g_app_launch_context, G_TYPE_OBJECT);
 
 /**
