@@ -920,9 +920,9 @@ g_local_file_query_filesystem_info (GFile         *file,
   gboolean no_size;
 #ifndef G_OS_WIN32
   guint64 block_size;
+  const char *fstype;
 #ifdef USE_STATFS
   struct statfs statfs_buffer;
-  const char *fstype;
 #elif defined(USE_STATVFS)
   struct statvfs statfs_buffer;
 #endif
@@ -972,7 +972,8 @@ g_local_file_query_filesystem_info (GFile         *file,
 
   attribute_matcher = g_file_attribute_matcher_new (attributes);
   
-  if (g_file_attribute_matcher_matches (attribute_matcher,
+  if (!no_size &&
+      g_file_attribute_matcher_matches (attribute_matcher,
 					G_FILE_ATTRIBUTE_FILESYSTEM_FREE))
     {
 #ifdef G_OS_WIN32
@@ -988,7 +989,8 @@ g_local_file_query_filesystem_info (GFile         *file,
       g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_FREE, block_size * statfs_buffer.f_bavail);
 #endif
     }
-  if (g_file_attribute_matcher_matches (attribute_matcher,
+  if (!no_size &&
+      g_file_attribute_matcher_matches (attribute_matcher,
 					G_FILE_ATTRIBUTE_FILESYSTEM_SIZE))
     {
 #ifdef G_OS_WIN32
@@ -1010,6 +1012,12 @@ g_local_file_query_filesystem_info (GFile         *file,
 #else
   fstype = get_fs_type (statfs_buffer.f_type);
 #endif
+
+#elif defined(USE_STATVFS) && defined(HAVE_STRUCT_STATVFS_F_BASETYPE)
+  fstype = g_strdup(statfs_buffer.f_basetype); 
+#endif
+
+#ifndef G_OS_WIN32
   if (fstype &&
       g_file_attribute_matcher_matches (attribute_matcher,
 					G_FILE_ATTRIBUTE_FILESYSTEM_TYPE))
@@ -1990,7 +1998,7 @@ g_local_file_make_directory (GFile         *file,
       else
 	g_set_error (error, G_IO_ERROR,
 		     g_io_error_from_errno (errsv),
-		     _("Error removing file: %s"),
+		     _("Error creating directory: %s"),
 		     g_strerror (errsv));
       return FALSE;
     }
