@@ -1,3 +1,5 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
+
 /* GIO - GLib Input, Output and Streaming Library
  * 
  * Copyright (C) 2006-2007 Red Hat, Inc.
@@ -56,7 +58,6 @@
 #endif /* HAVE_XATTR */
 
 #include <glib/gstdio.h>
-#include <glib/gchecksum.h>
 #include <gfileattribute-priv.h>
 
 #include "glibintl.h"
@@ -97,20 +98,24 @@ struct ThumbMD5Context {
 	guint32 bits[2];
 	unsigned char in[64];
 };
+
 #ifndef G_OS_WIN32
+
 typedef struct {
   char *user_name;
   char *real_name;
 } UidData;
-#endif
+
 G_LOCK_DEFINE_STATIC (uid_cache);
 static GHashTable *uid_cache = NULL;
 
 G_LOCK_DEFINE_STATIC (gid_cache);
 static GHashTable *gid_cache = NULL;
 
+#endif  /* !G_OS_WIN32 */
+
 char *
-_g_local_file_info_create_etag (struct stat *statbuf)
+_g_local_file_info_create_etag (GLocalFileStat *statbuf)
 {
   GTimeVal tv;
   
@@ -127,7 +132,7 @@ _g_local_file_info_create_etag (struct stat *statbuf)
 }
 
 static char *
-_g_local_file_info_create_file_id (struct stat *statbuf)
+_g_local_file_info_create_file_id (GLocalFileStat *statbuf)
 {
   return g_strdup_printf ("l%" G_GUINT64_FORMAT ":%" G_GUINT64_FORMAT,
 			  (guint64) statbuf->st_dev, 
@@ -135,13 +140,15 @@ _g_local_file_info_create_file_id (struct stat *statbuf)
 }
 
 static char *
-_g_local_file_info_create_fs_id (struct stat *statbuf)
+_g_local_file_info_create_fs_id (GLocalFileStat *statbuf)
 {
   return g_strdup_printf ("l%" G_GUINT64_FORMAT,
 			  (guint64) statbuf->st_dev);
 }
 
 
+#ifdef S_ISLNK
+
 static gchar *
 read_link (const gchar *full_name)
 {
@@ -175,6 +182,8 @@ read_link (const gchar *full_name)
 #endif
 }
 
+#endif  /* S_ISLNK */
+
 /* Get the SELinux security context */
 static void
 get_selinux_context (const char            *path,
@@ -724,13 +733,13 @@ set_xattr (char                       *filename,
 
   if (g_str_has_prefix (escaped_attribute, "xattr::"))
     {
-      escaped_attribute += 6;
+      escaped_attribute += strlen ("xattr::");
       is_user = TRUE;
     }
   else
     {
       g_warn_if_fail (g_str_has_prefix (escaped_attribute, "xattr-sys::"));
-      escaped_attribute += 10;
+      escaped_attribute += strlen ("xattr-sys::");
       is_user = FALSE;
     }
   
@@ -759,7 +768,7 @@ set_xattr (char                       *filename,
       g_set_error (error, G_IO_ERROR,
 		   g_io_error_from_errno (errsv),
 		   _("Error setting extended attribute '%s': %s"),
-		   escaped_attribute, g_strerror (errno));
+		   escaped_attribute, g_strerror (errsv));
       return FALSE;
     }
   
@@ -774,11 +783,15 @@ _g_local_file_info_get_parent_info (const char            *dir,
 				    GFileAttributeMatcher *attribute_matcher,
 				    GLocalParentFileInfo  *parent_info)
 {
+  /* Use plain struct stat for now as long as we only look at the
+   * S_ISVTX bit which doesn't exist on Win32 anyway.
+   */
   struct stat statbuf;
   int res;
   
   parent_info->writable = FALSE;
   parent_info->is_sticky = FALSE;
+  parent_info->has_trash_dir = FALSE;
   parent_info->device = 0;
 
   if (g_file_attribute_matcher_matches (attribute_matcher, G_FILE_ATTRIBUTE_ACCESS_CAN_RENAME) ||
@@ -786,6 +799,11 @@ _g_local_file_info_get_parent_info (const char            *dir,
       g_file_attribute_matcher_matches (attribute_matcher, G_FILE_ATTRIBUTE_ACCESS_CAN_TRASH) ||
       g_file_attribute_matcher_matches (attribute_matcher, G_FILE_ATTRIBUTE_UNIX_IS_MOUNTPOINT))
     {
+      /* FIXME: Windows: The underlying _waccess() call in the C
+       * library is mostly pointless as it only looks at the READONLY
+       * FAT-style attribute of the file, it doesn't check the ACL at
+       * all.
+       */
       parent_info->writable = (g_access (dir, W_OK) == 0);
       
       res = g_stat (dir, &statbuf);
@@ -804,6 +822,10 @@ _g_local_file_info_get_parent_info (const char            *dir,
 #endif
 	  parent_info->owner = statbuf.st_uid;
 	  parent_info->device = statbuf.st_dev;
+          /* No need to find trash dir if its not writable anyway */
+          if (parent_info->writable &&
+              g_file_attribute_matcher_matches (attribute_matcher, G_FILE_ATTRIBUTE_ACCESS_CAN_TRASH))
+            parent_info->has_trash_dir = _g_local_file_has_trash_dir (dir, statbuf.st_dev);
 	}
     }
 }
@@ -812,9 +834,10 @@ static void
 get_access_rights (GFileAttributeMatcher *attribute_matcher,
 		   GFileInfo             *info,
 		   const gchar           *path,
-		   struct stat           *statbuf,
+		   GLocalFileStat        *statbuf,
 		   GLocalParentFileInfo  *parent_info)
 {
+  /* FIXME: Windows: The underlyin _waccess() is mostly pointless */
   if (g_file_attribute_matcher_matches (attribute_matcher,
 					G_FILE_ATTRIBUTE_ACCESS_CAN_READ))
     g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ,
@@ -861,16 +884,15 @@ get_access_rights (GFileAttributeMatcher *attribute_matcher,
 	g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_DELETE,
 					   writable);
 
-      /* TODO: This means we can move it, but we should also look for a trash dir */
       if (g_file_attribute_matcher_matches (attribute_matcher, G_FILE_ATTRIBUTE_ACCESS_CAN_TRASH))
-	g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_TRASH,
-					   writable);
+        g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_TRASH,
+                                           writable && parent_info->has_trash_dir);
     }
 }
 
 static void
 set_info_from_stat (GFileInfo             *info, 
-                    struct stat           *statbuf,
+                    GLocalFileStat        *statbuf,
 		    GFileAttributeMatcher *attribute_matcher)
 {
   GFileType file_type;
@@ -900,12 +922,16 @@ set_info_from_stat (GFileInfo             *info,
   g_file_info_set_size (info, statbuf->st_size);
 
   g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_DEVICE, statbuf->st_dev);
+#ifndef G_OS_WIN32
+  /* Pointless setting these on Windows even if they exist in the struct */
   g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_UNIX_INODE, statbuf->st_ino);
-  g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_MODE, statbuf->st_mode);
   g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_NLINK, statbuf->st_nlink);
   g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_UID, statbuf->st_uid);
   g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_GID, statbuf->st_gid);
   g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_RDEV, statbuf->st_rdev);
+#endif
+  /* FIXME: st_mode is mostly pointless on Windows, too. Set the attribute or not? */
+  g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_MODE, statbuf->st_mode);
 #if defined (HAVE_STRUCT_STAT_ST_BLKSIZE)
   g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_BLOCK_SIZE, statbuf->st_blksize);
 #endif
@@ -959,6 +985,8 @@ set_info_from_stat (GFileInfo             *info,
     }
 }
 
+#ifndef G_OS_WIN32
+
 static char *
 make_valid_utf8 (const char *name)
 {
@@ -1013,7 +1041,7 @@ convert_pwd_string_to_utf8 (char *pwd_str)
   
   return utf8_string;
 }
-#ifndef G_OS_WIN32
+
 static void
 uid_data_free (UidData *data)
 {
@@ -1160,12 +1188,13 @@ get_groupname_from_gid (gid_t gid)
   G_UNLOCK (gid_cache);
   return res;
 }
+
 #endif /* !G_OS_WIN32 */
 
 static char *
 get_content_type (const char          *basename,
 		  const char          *path,
-		  struct stat         *statbuf,
+		  GLocalFileStat      *statbuf,
 		  gboolean             is_symlink,
 		  gboolean             symlink_broken,
 		  GFileQueryInfoFlags  flags,
@@ -1213,7 +1242,7 @@ get_content_type (const char          *basename,
 	      
 	      res = read (fd, sniff_buffer, sniff_length);
 	      close (fd);
-	      if (res > 0)
+	      if (res >= 0)
 		{
 		  g_free (content_type);
 		  content_type = g_content_type_guess (basename, sniff_buffer, res, NULL);
@@ -1369,10 +1398,15 @@ _g_local_file_info_get (const char             *basename,
 			GError                **error)
 {
   GFileInfo *info;
-  struct stat statbuf;
+  GLocalFileStat statbuf;
+#ifdef S_ISLNK
   struct stat statbuf2;
+#endif
   int res;
   gboolean is_symlink, symlink_broken;
+#ifdef G_OS_WIN32
+  DWORD dos_attributes;
+#endif
 
   info = g_file_info_new ();
 
@@ -1385,14 +1419,43 @@ _g_local_file_info_get (const char             *basename,
   if (attribute_matcher == NULL)
     return info;
 
+#ifndef G_OS_WIN32
   res = g_lstat (path, &statbuf);
+#else
+  {
+    wchar_t *wpath = g_utf8_to_utf16 (path, -1, NULL, NULL, error);
+    int len;
+
+    if (wpath == NULL)
+      {
+        g_object_unref (info);
+        return NULL;
+      }
+
+    len = wcslen (wpath);
+    while (len > 0 && G_IS_DIR_SEPARATOR (wpath[len-1]))
+      len--;
+    if (len > 0 &&
+        (!g_path_is_absolute (path) || len > g_path_skip_root (path) - path))
+      wpath[len] = '\0';
+
+    res = _wstati64 (wpath, &statbuf);
+    dos_attributes = GetFileAttributesW (wpath);
+
+    g_free (wpath);
+  }
+#endif
+
   if (res == -1)
     {
+      int errsv = errno;
+      char *display_name = g_filename_display_name (path);
       g_object_unref (info);
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error stating file '%s': %s"),
-		   path, g_strerror (errno));
+		   display_name, g_strerror (errsv));
+      g_free (display_name);
       return NULL;
     }
   
@@ -1403,6 +1466,7 @@ _g_local_file_info_get (const char             *basename,
 #endif
   symlink_broken = FALSE;
   
+#ifdef S_ISLNK
   if (is_symlink)
     {
       g_file_info_set_is_symlink (info, TRUE);
@@ -1419,15 +1483,28 @@ _g_local_file_info_get (const char             *basename,
 	    symlink_broken = TRUE;
 	}
     }
+#endif
 
   set_info_from_stat (info, &statbuf, attribute_matcher);
   
+#ifndef G_OS_WIN32
   if (basename != NULL && basename[0] == '.')
     g_file_info_set_is_hidden (info, TRUE);
 
   if (basename != NULL && basename[strlen (basename) -1] == '~')
     g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_STANDARD_IS_BACKUP, TRUE);
+#else
+  if (dos_attributes & FILE_ATTRIBUTE_HIDDEN)
+    g_file_info_set_is_hidden (info, TRUE);
 
+  if (dos_attributes & FILE_ATTRIBUTE_ARCHIVE)
+    g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_DOS_IS_ARCHIVE, TRUE);
+
+  if (dos_attributes & FILE_ATTRIBUTE_SYSTEM)
+    g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_DOS_IS_SYSTEM, TRUE);
+#endif
+
+#ifdef S_ISLNK
   if (is_symlink &&
       g_file_attribute_matcher_matches (attribute_matcher,
 					G_FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET))
@@ -1436,6 +1513,7 @@ _g_local_file_info_get (const char             *basename,
       g_file_info_set_symlink_target (info, link);
       g_free (link);
     }
+#endif
 
   if (g_file_attribute_matcher_matches (attribute_matcher,
 					G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME))
@@ -1485,48 +1563,36 @@ _g_local_file_info_get (const char             *basename,
 	  if (g_file_attribute_matcher_matches (attribute_matcher,
 						G_FILE_ATTRIBUTE_STANDARD_ICON))
 	    {
-	      char *mimetype_icon, *generic_mimetype_icon, *type_icon, *p;
-	      char *icon_names[3];
 	      GIcon *icon;
-	      int i;
 
-	      mimetype_icon = g_strdup (content_type);
-	      
-	      while ((p = strchr (mimetype_icon, '/')) != NULL)
-		*p = '-';
+              icon = g_content_type_get_icon (content_type);
+              if (icon != NULL)
+                {
+                  if (G_IS_THEMED_ICON (icon))
+                    {
+                      const char *preferred_icon = NULL;
+                      const char *type_icon = NULL;
 
-	      p = strchr (content_type, '/');
-	      if (p == NULL)
-		p = content_type + strlen (content_type);
+                      if (strcmp (path, g_get_home_dir ()) == 0)
+                        preferred_icon = "user-home";
+                      else if (strcmp (path, g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP)) == 0) 
+                        preferred_icon = "user-desktop";
+                      if (S_ISDIR (statbuf.st_mode)) 
+                        type_icon = "folder";
+                      else if (statbuf.st_mode & S_IXUSR)
+                        type_icon = "application-x-executable";
+                      else
+                        type_icon = "text-x-generic";
 
-	      generic_mimetype_icon = g_malloc (p - content_type + strlen ("-x-generic") + 1);
-	      memcpy (generic_mimetype_icon, content_type, p - content_type);
-	      memcpy (generic_mimetype_icon + (p - content_type), "-x-generic", strlen ("-x-generic"));
-	      generic_mimetype_icon[(p - content_type) + strlen ("-x-generic")] = 0;
+                      if (preferred_icon)
+                        g_themed_icon_prepend_name (G_THEMED_ICON (icon), preferred_icon);
+                      if (type_icon) 
+                        g_themed_icon_append_name (G_THEMED_ICON (icon), type_icon);
+                    }
 
-	      /* TODO: Special case desktop dir? That could be expensive with xdg dirs... */
-	      if (strcmp (path, g_get_home_dir ()) == 0)
-		type_icon = "user-home";
-	      else if (S_ISDIR (statbuf.st_mode)) 
-		type_icon = "folder";
-	      else if (statbuf.st_mode & S_IXUSR)
-		type_icon = "application-x-executable";
-	      else
-		type_icon = "text-x-generic";
-
-	      i = 0;
-	      icon_names[i++] = mimetype_icon;
-	      icon_names[i++] = generic_mimetype_icon;
-	      if (strcmp (generic_mimetype_icon, type_icon) != 0 &&
-		  strcmp (mimetype_icon, type_icon) != 0) 
-		icon_names[i++] = type_icon;
-	      
-	      icon = g_themed_icon_new_from_names (icon_names, i);
-	      g_file_info_set_icon (info, icon);
-	      
-	      g_object_unref (icon);
-	      g_free (mimetype_icon);
-	      g_free (generic_mimetype_icon);
+                  g_file_info_set_icon (info, icon);
+                  g_object_unref (icon);
+                }
 	    }
 	  
 	  g_free (content_type);
@@ -1613,16 +1679,24 @@ _g_local_file_info_get_from_fd (int      fd,
 				char    *attributes,
 				GError **error)
 {
-  struct stat stat_buf;
+  GLocalFileStat stat_buf;
   GFileAttributeMatcher *matcher;
   GFileInfo *info;
   
-  if (fstat (fd, &stat_buf) == -1)
+#ifdef G_OS_WIN32
+#define FSTAT _fstati64
+#else
+#define FSTAT fstat
+#endif
+
+  if (FSTAT (fd, &stat_buf) == -1)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error stating file descriptor: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return NULL;
     }
 
@@ -1675,6 +1749,7 @@ get_uint32 (const GFileAttributeValue  *value,
   return TRUE;
 }
 
+#ifdef HAVE_UTIMES
 static gboolean
 get_uint64 (const GFileAttributeValue  *value,
 	    guint64                    *val_out,
@@ -1691,6 +1766,7 @@ get_uint64 (const GFileAttributeValue  *value,
   
   return TRUE;
 }
+#endif
 
 #if defined(HAVE_SYMLINK)
 static gboolean
@@ -1723,10 +1799,12 @@ set_unix_mode (char                       *filename,
   
   if (g_chmod (filename, val) == -1)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error setting permissions: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return FALSE;
     }
   return TRUE;
@@ -1772,10 +1850,12 @@ set_unix_uid_gid (char                       *filename,
   
   if (res == -1)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error setting owner: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
 	  return FALSE;
     }
   return TRUE;
@@ -1803,10 +1883,12 @@ set_symlink (char                       *filename,
   
   if (g_lstat (filename, &statbuf))
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error setting symlink: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return FALSE;
     }
   
@@ -1820,19 +1902,23 @@ set_symlink (char                       *filename,
   
   if (g_unlink (filename))
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error setting symlink: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return FALSE;
     }
   
   if (symlink (filename, val) != 0)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error setting symlink: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return FALSE;
     }
   
@@ -1840,6 +1926,7 @@ set_symlink (char                       *filename,
 }
 #endif
 
+#ifdef HAVE_UTIMES
 static int
 lazy_stat (char        *filename, 
            struct stat *statbuf, 
@@ -1859,7 +1946,6 @@ lazy_stat (char        *filename,
 }
 
 
-#ifdef HAVE_UTIMES
 static gboolean
 set_mtime_atime (char                       *filename,
 		 const GFileAttributeValue  *mtime_value,
@@ -1932,10 +2018,12 @@ set_mtime_atime (char                       *filename,
   res = utimes (filename, times);
   if (res == -1)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error setting owner: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
 	  return FALSE;
     }
   return TRUE;
@@ -2000,9 +2088,16 @@ _g_local_file_info_set_attributes  (char                 *filename,
 				    GCancellable         *cancellable,
 				    GError              **error)
 {
-  GFileAttributeValue *value, *uid, *gid;
+  GFileAttributeValue *value;
+#ifdef HAVE_CHOWN
+  GFileAttributeValue *uid, *gid;
+#endif
+#ifdef HAVE_UTIMES
   GFileAttributeValue *mtime, *mtime_usec, *atime, *atime_usec;
+#endif
+#if defined (HAVE_CHOWN) && defined (HAVE_UTIMES)
   GFileAttributeStatus status;
+#endif
   gboolean res;
   
   /* Handles setting multiple specified data in a single set, and takes care
