@@ -521,12 +521,14 @@ print_entry (GOptionGroup       *group,
 }
 
 static gboolean
-group_has_visible_entries (GOptionGroup *group,
+group_has_visible_entries (GOptionContext *context,
+                           GOptionGroup *group,
                            gboolean      main_entries)
 {
   GOptionFlags reject_filter = G_OPTION_FLAG_HIDDEN;
   GOptionEntry *entry;
   gint i, l;
+  gboolean main_group = group == context->main_group;
 
   if (!main_entries)
     reject_filter |= G_OPTION_FLAG_IN_MAIN;
@@ -535,7 +537,7 @@ group_has_visible_entries (GOptionGroup *group,
     {
       entry = &group->entries[i];
 
-      if (main_entries && !(entry->flags & G_OPTION_FLAG_IN_MAIN))
+      if (main_entries && !main_group && !(entry->flags & G_OPTION_FLAG_IN_MAIN))
         continue;
       if (!(entry->flags & reject_filter))
         return TRUE;
@@ -545,12 +547,13 @@ group_has_visible_entries (GOptionGroup *group,
 }
 
 static gboolean
-group_list_has_visible_entires (GList    *group_list,
-                                gboolean  main_entries)
+group_list_has_visible_entires (GOptionContext *context,
+                                GList          *group_list,
+                                gboolean       main_entries)
 {
   while (group_list)
     {
-      if (group_has_visible_entries (group_list->data, main_entries))
+      if (group_has_visible_entries (context, group_list->data, main_entries))
         return TRUE;
 
       group_list = group_list->next;
@@ -726,7 +729,7 @@ g_option_context_get_help (GOptionContext *context,
 	{
 	  GOptionGroup *group = list->data;
 
-	  if (group_has_visible_entries (group, FALSE))
+	  if (group_has_visible_entries (context, group, FALSE))
 	    g_string_append_printf (string, "  --help-%-*s %s\n",
 				    max_length - 5, group->name,
 				    TRANSLATE (group, group->help_description));
@@ -741,7 +744,7 @@ g_option_context_get_help (GOptionContext *context,
     {
       /* Print a certain group */
 
-      if (group_has_visible_entries (group, FALSE))
+      if (group_has_visible_entries (context, group, FALSE))
         {
           g_string_append (string, TRANSLATE (group, group->description));
           g_string_append (string, "\n");
@@ -760,7 +763,7 @@ g_option_context_get_help (GOptionContext *context,
 	{
 	  GOptionGroup *group = list->data;
 
-	  if (group_has_visible_entries (group, FALSE))
+	  if (group_has_visible_entries (context, group, FALSE))
 	    {
 	      g_string_append (string, group->description);
 	      g_string_append (string, "\n");
@@ -777,8 +780,8 @@ g_option_context_get_help (GOptionContext *context,
   
   /* Print application options if --help or --help-all has been specified */
   if ((main_help || !group) &&
-      (group_has_visible_entries (context->main_group, TRUE) ||
-       group_list_has_visible_entires (context->groups, TRUE)))
+      (group_has_visible_entries (context, context->main_group, TRUE) ||
+       group_list_has_visible_entires (context, context->groups, TRUE)))
     {
       list = context->groups;
 
@@ -2055,7 +2058,7 @@ static gchar *
 dgettext_swapped (const gchar *msgid, 
 		  const gchar *domainname)
 {
-  return dgettext (domainname, msgid);
+  return g_dgettext (domainname, msgid);
 }
 
 /**
