@@ -38,6 +38,7 @@
 #include <wchar.h>
 #include <direct.h>
 #include <io.h>
+#include <sys/utime.h>
 #endif
 
 #include "gstdio.h"
@@ -84,7 +85,11 @@ g_access (const gchar *filename,
       return -1;
     }
 
-  retval = _waccess (wfilename, mode);
+#ifndef X_OK
+#define X_OK 1
+#endif
+
+  retval = _waccess (wfilename, mode & ~X_OK);
   save_errno = errno;
 
   g_free (wfilename);
@@ -197,7 +202,7 @@ g_open (const gchar *filename,
  *
  * A wrapper for the POSIX creat() function. The creat() function is
  * used to convert a pathname into a file descriptor, creating a file
- * if necessar. Note that on POSIX systems file descriptors are
+ * if necessary. Note that on POSIX systems file descriptors are
  * implemented by the operating system. On Windows, it's the C library
  * that implements creat() and file descriptors. The actual Windows
  * API for opening files is something different.
@@ -725,5 +730,48 @@ g_freopen (const gchar *filename,
 #endif
 }
 
+/**
+ * g_utime:
+ * @filename: a pathname in the GLib file name encoding (UTF-8 on Windows)
+ * @utb: a pointer to a struct utimbuf.
+ *
+ * A wrapper for the POSIX utime() function. The utime() function
+ * sets the access and modification timestamps of a file.
+ * 
+ * See your C library manual for more details about how utime() works
+ * on your system.
+ *
+ * Returns: 0 if the operation was successful, -1 if an error 
+ *    occurred
+ * 
+ * Since: 2.18
+ */
+int
+g_utime (const gchar    *filename,
+	 struct utimbuf *utb)
+{
+#ifdef G_OS_WIN32
+  wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
+  int retval;
+  int save_errno;
+
+  if (wfilename == NULL)
+    {
+      errno = EINVAL;
+      return -1;
+    }
+  
+  retval = _wutime (wfilename, (struct _utimbuf*) utb);
+  save_errno = errno;
+
+  g_free (wfilename);
+
+  errno = save_errno;
+  return retval;
+#else
+  return utime (filename, utb);
+#endif
+}
+
 #define __G_STDIO_C__
 #include "galiasdef.c"
